/* Main function for root server.
   Copyright (C) 2003 Free Software Foundation, Inc.
   This file is part of the GNU Hurd.

   The GNU Hurd is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU Hurd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.  */

#include <stdio.h>
#include <l4.h>

void cls (void);

void
print_thread (l4_thread_id_t thread)
{
  if (l4_is_nil_thread (thread))
    printf ("nilthread");
  else if (l4_is_local_id (thread))
    printf ("Local 0x%x", thread.raw);
  else
    printf ("No 0x%x Version 0x%x", l4_thread_no (thread),
	    l4_version (thread));
}


void
print_thread_info (l4_thread_id_t thread)
{
  l4_thread_id_t dest = thread;
  l4_word_t control = 0;
  l4_word_t sp = 0;
  l4_word_t ip = 0;
  l4_word_t flags = 0;
  l4_word_t user_handle = 0;
  l4_thread_id_t pager = l4_nilthread;

  printf ("Thread ID: ");
  print_thread (l4_global_id_of (thread));
  printf (" ");
  print_thread (l4_local_id_of (thread));
  printf ("\n");

  printf ("Pager ID: ");
  print_thread (l4_pager_of (thread));
  printf ("\n");

  printf ("User Defined Handle: 0x%x\n", l4_user_defined_handle_of (thread));

  l4_exchange_registers (&dest, &control, &sp, &ip, &flags,
			 &user_handle, &pager);
  printf ("SP: 0x%x  IP: 0x%x  FLAGS: 0x%x\n", sp, ip, flags);
  printf ("State: %s, %s, %s\n",
	  (control & L4_XCHG_REGS_HALTED) ? "halted" : "running",
	  (control & L4_XCHG_REGS_SENDING) ? "sending" : "not sending",
	  (control & L4_XCHG_REGS_RECEIVING)
	  ? "receiving" : "not receiving");	   
}



#define STACK_SIZE 4096
char pager_stack[STACK_SIZE];

void
pager (void)
{
  print_thread_info (l4_myself ());
  while (1)
    l4_yield ();
}


int
main (void)
{
  l4_thread_id_t thread;
  l4_word_t result;
  void *utcb;

  cls ();
  print_thread_info (l4_myself ());

  /* FIXME: This is not specified by the standard.  We don't know
     where and how much space we have for other thread's UTCB
     areas in the root server.  */
  utcb = (void *) ((l4_my_local_id ().raw & ~(l4_utcb_size () - 1))
    + l4_utcb_size ());
  thread = l4_global_id (l4_thread_no (l4_myself ()) + 1, 1);

  printf ("Creating new thread at utcb 0x%x: ", (l4_word_t) utcb);
  result = l4_thread_control (thread, l4_myself (), l4_myself (),
			      l4_nilthread, utcb);
  printf ("%s\n", result ? "successful" : "failed");
  printf ("Starting new thread with esp 0x%x at eip 0x%x\n",
	  ((l4_word_t) pager_stack) + sizeof (pager_stack),
	  (l4_word_t) pager);
  /* Set the priority of the other thread to our priority.  Otherwise
     it is 100 and will never be scheduled as long as we are not in a
     blocking receive.  */
  l4_set_priority (thread, 255);
  l4_start_sp_ip (thread, ((l4_word_t) pager_stack) + sizeof (pager_stack),
		  (l4_word_t) pager);

  while (1)
    l4_yield ();

  return 0;
}
