/* Main function for root server.
   Copyright (C) 2003 Free Software Foundation, Inc.
   This file is part of the GNU Hurd.

   The GNU Hurd is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU Hurd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.  */

#include <stdio.h>
#include <l4.h>

void cls (void);

int
main (void)
{
  l4_kip_t kip;
  l4_api_version_t version;
  l4_api_flags_t flags;
  l4_kernel_id_t id;
  l4_word_t ver;
  l4_word_t subver;
  l4_word_t subsubver;
  char *supplier;
  l4_word_t year;
  l4_word_t month;
  l4_word_t day;
  char *feature;
  l4_page_info_t page;
  int nr;

  cls ();

  kip = l4_get_kernel_interface ();

  printf ("Kernel Interface Page: 0x%x/%u (%c%c%c%c)\n",
	  (unsigned int) kip, l4_kip_area_size_log2 (),
	  kip->magic[0], kip->magic[1], kip->magic[2], kip->magic[3]);
  printf ("API Version: ");

  version = l4_api_version ();
  if (version.version == 0x02)
    printf ("Version 2");
  else if (version.version == 0x83 && version.subversion == 0x80)
    printf ("Experimental Version X.0");
  else if (version.version == 0x83 && version.subversion == 0x81)
    printf ("Experimental Version X.1");
  else if (version.version == 0x84)
    printf ("Experimental Version X.2 rev %u", version.subversion);
  else if (version.version == 0x04)
    printf ("Version 4 rev %u", version.subversion);
  else if (version.version & 0x80)
    printf ("Unknown Experimental Version");
  else
    printf ("Unknown Version");
  printf (" (0x%x, 0x%x)\n", version.version, version.subversion);

  flags = l4_api_flags ();
  printf ("API Flags: ");
  switch (flags.endian)
    {
    case L4_API_FLAGS_LITTLE_ENDIAN:
      printf ("little endian, ");
      break;

    case L4_API_FLAGS_BIG_ENDIAN:
      printf ("big endian, ");
      break;

    default:
      printf ("unknown endian, ");
      break;
    }
  switch (flags.endian)
    {
    case L4_API_FLAGS_WORDSIZE_32:
      printf ("32-bit\n");
      break;

    case L4_API_FLAGS_WORDSIZE_64:
      printf ("64-bit\n");
      break;

    default:
      printf ("unknown wordsize\n");
      break;
    }

  printf ("Kernel: ");
  id = l4_kernel_id ();
  if (id.id == 0 && id.subid == 1)
    printf ("L4/486");
  else if (id.id == 0 && id.subid == 2)
    printf ("L4/Pentium");
  else if (id.id == 0 && id.subid == 3)
    printf ("L4/x86");
  else if (id.id == 1 && id.subid == 1)
    printf ("L4/Mips");
  else if (id.id == 2 && id.subid == 1)
    printf ("L4/Alpha");
  else if (id.id == 3 && id.subid == 1)
    printf ("Fiasco");
  else if (id.id == 4 && id.subid == 1)
    printf ("L4Ka/Hazelnut");
  else if (id.id == 4 && id.subid == 2)
    printf ("L4Ka/Pistachio");
  else if (id.id == 4 && id.subid == 3)
    printf ("L4Ka/Strawberry");
  else
    printf ("Unknown Kernel");
  printf (" from ");

  supplier = l4_kernel_supplier ();
  if (supplier[0] == 'G' && supplier[1] == 'M'
      && supplier[2] == 'D' && supplier[3] == ' ')
    printf ("GMD");
  else if (supplier[0] == 'I' && supplier[1] == 'B'
      && supplier[2] == 'M' && supplier[3] == ' ')
    printf ("IBM Research");
  else if (supplier[0] == 'U' && supplier[1] == 'N'
      && supplier[2] == 'S' && supplier[3] == 'W')
    printf ("University of New South Wales, Sydney");
  else if (supplier[0] == 'T' && supplier[1] == 'U'
      && supplier[2] == 'D' && supplier[3] == ' ')
    printf ("Technische Universitaet Dresden");
  else if (supplier[0] == 'U' && supplier[1] == 'K'
      && supplier[2] == 'a' && supplier[3] == ' ')
    printf ("Universitaet Karlsruhe (TH)");
  else
    printf ("Unknown Supplier");
  printf (" (0x%x, 0x%x, `%c%c%c%c')\n",
	  id.id, id.subid, supplier[0], supplier[1], supplier[2], supplier[3]);

  l4_kernel_gen_date (&year, &month, &day);
  l4_kernel_version (&ver, &subver, &subsubver);
  printf ("        Version %u.%u.%u, generated %u/%u/%u\n",
	  ver, subver, subsubver, year, month, day);
  /* The kernel version string is too long and stretches over two
     precious lines.  */
#if 0
  printf ("        %s\n", l4_kernel_version_string);
#endif
  feature = l4_feature (0);
  if (feature)
    {
      printf ("        [");
      nr = 1;
      do
	{
	  printf ("%s", feature);
	  feature = l4_feature (nr++);
	  if (feature)
	    printf (", ");
	}
      while (feature);
      printf ("]\n");
    }
  printf ("Processor: %u (", l4_num_processors ());
  for (nr = 0; nr < l4_num_processors (); nr++)
    {
      l4_proc_desc_t proc = l4_proc_desc (nr);
      printf ("I %u kHz E %u kHz", l4_proc_internal_freq (proc),
	      l4_proc_external_freq (proc));
      if (nr + 1 < l4_num_processors ())
	printf (", ");
    }
  printf (")\n");

  printf ("Clock: Read precision %u, Schedule precision %u\n",
	  l4_read_precision (), l4_schedule_precision ());

  page = l4_page_rights ();
  printf ("Page Info: Access %c%c%c, Sizes: ",
	  page.read ? 'r' : '-',
	  page.write ? 'w' : '-',
	  page.execute ? 'x' : '-');
  for (nr = 0; nr < sizeof (l4_word_t) * 8; nr++)
    if (l4_page_size_mask () & (1 << nr))
      printf ("%u ", nr);
  printf ("\n");

  printf ("Threads: User Base %u, System Base %u, Thread No Bits %u\n",
	  l4_thread_user_base (), l4_thread_system_base (),
	  l4_thread_id_bits ());

  printf ("UTCB Info: Minimum Area Size 0x%x, Alignment Log2 %u, Size 0x%x\n",
	  l4_utcb_area_size (), l4_utcb_alignment_log2 (),
	  l4_utcb_size ());

  printf ("Memory: %u\n", l4_num_memory_desc ());
  for (nr = 0; nr < l4_num_memory_desc (); nr++)
    {
      l4_memory_desc_t mem = l4_memory_desc (nr);
      l4_word_t type;

      printf ("        0x%x - 0x%x %s ",
	      l4_memory_desc_low (mem), l4_memory_desc_high (mem),
	      l4_is_memory_desc_virtual (mem) ? "virtual " : "physical");

      type = l4_memory_desc_type (mem) & L4_MEMDESC_MASK;

      if (type == L4_MEMDESC_UNDEFINED)
	printf ("undefined");
      else if (type == L4_MEMDESC_CONVENTIONAL)
	printf ("conventional");
      else if (type == L4_MEMDESC_RESERVED)
	printf ("reserved");
      else if (type == L4_MEMDESC_DEDICATED)
	printf ("dedicated");
      else if (type == L4_MEMDESC_SHARED)
	printf ("shared");
      else if (type == L4_MEMDESC_BOOTLOADER)
	printf ("boot loader 0x%x", l4_memory_desc_type (mem) >> 4);
      else if (type == L4_MEMDESC_ARCH)
	printf ("architecture 0x%x", l4_memory_desc_type (mem) >> 4);
      else
	printf ("unknown");
      printf ("\n");
      mem++;
    }

  printf ("Boot Info: 0x%x\n", l4_boot_info ());

  return 0;
}
